import type * as Context from 'effect/Context';
import * as Effect from 'effect/Effect';
import * as Stream from 'effect/Stream';
import type * as Types from 'effect/Types';
import { type Versioned } from './Versioned.js';
export declare const ComputedTypeId: unique symbol;
export type ComputedTypeId = typeof ComputedTypeId;
export interface Computed<in out A, out E = never, out R = never> extends Versioned<A, E, R> {
    readonly [ComputedTypeId]: Computed.Variance<A, E, R>;
}
export declare namespace Computed {
    type Variance<out A, out E, out R> = {
        readonly _A: Types.Covariant<A>;
        readonly _E: Types.Covariant<E>;
        readonly _R: Types.Covariant<R>;
    };
    type Success<T> = [T] extends [
        {
            readonly [ComputedTypeId]: Variance<infer A, infer _, infer __>;
        }
    ] ? A : never;
    type Error<T> = [T] extends [
        {
            readonly [ComputedTypeId]: Variance<infer _, infer E, infer __>;
        }
    ] ? E : never;
    type Context<T> = [T] extends [
        {
            readonly [ComputedTypeId]: Variance<infer _, infer __, infer R>;
        }
    ] ? R : never;
}
/**
 * @internal
 */
export declare function makeComputed<A, E, R>(get: Effect.Effect<A, E, R>, changes: Stream.Stream<A, E, R>, version: Effect.Effect<number, never, R>): Computed<A, E, R>;
/**
 * @internal
 */
export declare function fromVersioned<A, E, R>(versioned: Versioned<A, E, R>): Computed<A, E, R>;
export declare function isComputed<A = unknown, E = unknown, R = unknown>(value: unknown): value is Computed<A, E, R>;
export declare function computedFromTag<Id, S, A, E, R>(tag: Context.Tag<Id, S>, f: (s: S) => Computed<A, E, R>): Computed<A, E, Id | R>;
