import type * as Context from 'effect/Context';
import * as Effect from 'effect/Effect';
import * as Layer from 'effect/Layer';
import * as Runtime from 'effect/Runtime';
import type * as Scope from 'effect/Scope';
import * as Stream from 'effect/Stream';
export type Provide<A, E, R> = ProvideContext<A> | ProvideLayer<A, E, R> | ProvideRuntime<A> | ProvideService<A, any> | ProvideServiceEffect<A, any, E, R>;
export interface ProvideContext<A> {
    readonly _tag: 'ProvideContext';
    readonly i0: Context.Context<A>;
}
export declare const ProvideContext: <A>(i0: Context.Context<A>) => ProvideContext<A>;
export interface ProvideLayer<A, E, R> {
    readonly _tag: 'ProvideLayer';
    readonly i0: Layer.Layer<A, E, R>;
}
export declare const ProvideLayer: <A, E, R>(i0: Layer.Layer<A, E, R>) => ProvideLayer<A, E, R>;
export interface ProvideService<I, S> {
    readonly _tag: 'ProvideService';
    readonly i0: Context.Tag<I, S>;
    readonly i1: S;
}
export declare const ProvideService: <I, S>(i0: Context.Tag<I, S>, i1: S) => ProvideService<I, S>;
export interface ProvideServiceEffect<I, S, E, R> {
    readonly _tag: 'ProvideServiceEffect';
    readonly i0: Context.Tag<I, S>;
    readonly i1: Effect.Effect<S, E, R>;
}
export declare const ProvideServiceEffect: <I, S, E, R>(i0: Context.Tag<I, S>, i1: Effect.Effect<S, E, R>) => ProvideServiceEffect<I, S, E, R>;
export interface ProvideRuntime<A> {
    readonly _tag: 'ProvideRuntime';
    readonly i0: Runtime.Runtime<A>;
}
export declare const ProvideRuntime: <A>(i0: Runtime.Runtime<A>) => ProvideRuntime<A>;
export declare function matchProvide<A, E, R, B>(self: Provide<A, E, R>, matchers: {
    readonly ProvideContext: (i0: Context.Context<A>) => B;
    readonly ProvideRuntime: (i0: Runtime.Runtime<A>) => B;
    readonly ProvideLayer: (i0: Layer.Layer<A, E, R>) => B;
    readonly ProvideService: (tag: Context.Tag<A, any>, service: any) => B;
    readonly ProvideServiceEffect: (tag: Context.Tag<A, any>, service: Effect.Effect<any, E, R>) => B;
}): B;
export declare function merge<A, E, R, B, E2, R2>(self: Provide<A, E, R>, that: Provide<B, E2, R2>): Provide<A | B, E | E2, Exclude<R, B> | R2>;
export declare function buildWithScope<A, E, R>(provide: Provide<A, E, R>, scope: Scope.Scope): Effect.Effect<Context.Context<A>, E, R>;
export declare function toLayer<A, E, R>(provide: Provide<A, E, R>): Layer.Layer<A, E, R>;
export declare function provideToEffect<A, E, R, R2 = never, E2 = never, S = never>(effect: Effect.Effect<A, E, R>, provide: Provide<S, E, R2>): Effect.Effect<A, E | E2, Exclude<R, S> | R2>;
export declare function provideToStream<A, E, R, R2 = never, E2 = never, S = never>(stream: Stream.Stream<A, E, R>, provide: Provide<S, E, R2>): Stream.Stream<A, E | E2, Exclude<R, S> | R2>;
export declare function runtimeToLayer<R>(runtime: Runtime.Runtime<R>): Layer.Layer<R>;
