import type { Channel } from 'effect/Channel';
import type { Chunk } from 'effect/Chunk';
import * as Effect from 'effect/Effect';
import * as Readable from 'effect/Readable';
import * as Stream from 'effect/Stream';
import * as Subscribable from 'effect/Subscribable';
import { EffectBase } from './internal.js';
export interface Versioned<A, E, R> extends Effect.Effect<A, E, R>, Subscribable.Subscribable<A, E, R> {
    readonly version: Effect.Effect<number, never, R>;
}
export declare namespace Versioned {
    type Success<T> = T extends Versioned<infer A, infer E, infer R> ? A : never;
    type Error<T> = T extends Versioned<infer A, infer E, infer R> ? E : never;
    type Context<T> = T extends Versioned<infer A, infer E, infer R> ? R : never;
}
export declare function make<A, E, R>(get: Effect.Effect<A, E, R>, changes: Stream.Stream<A, E, R>, version: Effect.Effect<number, never, R>): Versioned<A, E, R>;
/**
 * @internal
 */
export declare class VersionedImpl<A, E, R> extends EffectBase<A, E, R> implements Versioned<A, E, R> {
    readonly changes: Stream.Stream<A, E, R>;
    readonly version: Effect.Effect<number, never, R>;
    readonly [Subscribable.TypeId]: typeof Subscribable.TypeId;
    readonly [Readable.TypeId]: typeof Readable.TypeId;
    readonly get: Effect.Effect<A, E, R>;
    readonly channel: Channel<Chunk<A>, unknown, E, unknown, unknown, unknown, R>;
    constructor(get: Effect.Effect<A, E, R>, changes: Stream.Stream<A, E, R>, version: Effect.Effect<number, never, R>);
    toEffect(): Effect.Effect<A, E, R>;
}
export declare function tuple<Versioneds extends ReadonlyArray<Versioned<any, any, any>>>(...versioneds: Versioneds): Versioned<{
    readonly [K in keyof Versioneds]: Versioned.Success<Versioneds[K]>;
}, Versioned.Error<Versioneds[keyof Versioneds]>, Versioned.Context<Versioneds[keyof Versioneds]>>;
export declare function struct<Versioneds extends Readonly<Record<string, Versioned<any, any, any>>>>(refs: Versioneds): Versioned<{
    readonly [K in keyof Versioneds]: Versioned.Success<Versioneds[K]>;
}, Versioned.Error<Versioneds[keyof Versioneds]>, Versioned.Context<Versioneds[keyof Versioneds]>>;
